--  rain.ex 
--  jiri babor
--  jbabor@paradise.net.nz
--  99-02-11
--  mods 16-Mar-04
--  color rain (bloodrain by Nemesis)
--  press esc to exit, any other key to change color

without warning
without type_check

include machine.e
include graphics.e

------------------------------------------------------------------
--  palette routines: uses code from ports.e by Jacques Deschenes
------------------------------------------------------------------

sequence icode,ocode
ocode = {#50,#52,#BA,0,0,0,0,#B0,#00,#EE,#5A,#58,#C3}
icode = {#50,#52,#BA,0,0,0,0,#EC,#BA,0,0,0,0,#88,#02,#5A,#58,#C3,#0}

atom iasm, inbyte, inport, oasm, outbyte, outport
integer n

oasm = allocate(length(ocode))
if not(oasm) then
    puts(1, "Error: memory allocation failed, ports initialization\n")
    abort( 1)
end if
poke(oasm, ocode)
outport = oasm + 3
outbyte = oasm + 8

n = length(icode)
iasm = allocate(n)
if not(iasm) then
    puts(1, "Error: memory allocation failed, ports initialization\n")
    abort( 1)
end if
poke(iasm, icode)
poke4(iasm + 9, iasm + n - 1)
inport = iasm + 3
inbyte = iasm + n - 1

procedure output(integer byte, integer port)
    poke4(outport, port)
    poke(outbyte, byte)
    call(oasm)
end procedure

function input(integer port)
    poke4(inport, port)
    call(iasm)
    return peek(inbyte)
end function

global procedure set_palettes(integer c, sequence s)
    -- set contiguous range of colors
    -- c is starting color index
    -- s is sequence of {r,g,b} palette entries
    -- rgb values 0..63
    output(c,#3C8)
    for i=1 to length(s) do
        output(s[i][1],#3C9)
        output(s[i][2],#3C9)
        output(s[i][3],#3C9)
    end for
end procedure
----------------------------------------------------------------


constant
    A = #A0000,         -- video memory
    dt = 0.005          -- 200 frames per second
object junk
sequence p,pal
atom m,t,t1,t2
integer a,c,co,dc,key


-- main ----------------------------------------------------------------------

p = repeat(repeat({0,0,0},128),6)   -- 6 different palettes
for i=1 to 128 do
    p[1][i][1] = floor((i-1)/2)
    p[2][i][2] = floor((i-1)/2)
    p[3][i][3] = floor((i-1)/2)
    p[4][i] = floor({(i-1)/3,(i-1)/3,0})
    p[5][i] = floor({(i-1)/3,0,(i-1)/3})
    p[6][i] = floor({0,(i-1)/3,(i-1)/3})
end for

junk = graphics_mode(19)
co=1
pal=p[co]
set_palettes(128,pal)
for x=0 to 319 do
    a = A+x
    c = 127+rand(128)
    dc = rand(3)
    for y=1 to 200 do
        poke(a,c)
        a += 320
        c += dc
        if c > 255 then c = 128 end if
    end for
end for

tick_rate(200)

m = 0
t = time()
t1 = t
t2 = t1+dt
while 1 do
    pal = prepend(pal[1..127],pal[128])
    set_palettes(128,pal)
    t1 = time()
    while t1 < t2 do t1 = time() end while
    t2 = t1+dt
    m += 1
    key = get_key()
    if key != -1 then
        if key = 27 then exit
        else
            co += 1
            if co > 6 then  co = 1 end if
            pal=p[co]
        end if
    end if
end while
t = time()-t

junk = graphics_mode(-1)
printf(1,"%5.2f fps\n",m/t)
